<?php

namespace Anyone\Onlyone\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Anyone\Onlyone\Http\Requests\MasterRequest;
use Anyone\Onlyone\Interfaces\MasterRepositoryInterface;
use Anyone\Onlyone\Models\Master;

/**
 * Resource controller class for master.
 */
class MasterResourceController extends BaseController
{

    /**
     * Initialize master resource controller.
     *
     * @param type MasterRepositoryInterface $master
     *
     * @return null
     */
    public function __construct(MasterRepositoryInterface $master)
    {
        parent::__construct();
        $this->repository = $master;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Anyone\Onlyone\Repositories\Criteria\MasterResourceCriteria::class);
    }

    /**
     * Display a list of master.
     *
     * @return Response
     */
    public function index(MasterRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Anyone\Onlyone\Repositories\Presenter\MasterPresenter::class)
                ->$function();
        }

        $masters = $this->repository->paginate();

        return $this->response->title(trans('onlyone::master.names'))
            ->view('onlyone::master.index', true)
            ->data(compact('masters'))
            ->output();
    }

    /**
     * Display master.
     *
     * @param Request $request
     * @param Model   $master
     *
     * @return Response
     */
    public function show(MasterRequest $request, Master $master)
    {

        if ($master->exists) {
            $view = 'onlyone::master.show';
        } else {
            $view = 'onlyone::master.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('onlyone::master.name'))
            ->data(compact('master'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new master.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(MasterRequest $request)
    {

        $master = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('onlyone::master.name')) 
            ->view('onlyone::master.create', true) 
            ->data(compact('master'))
            ->output();
    }

    /**
     * Create new master.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(MasterRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $master                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('onlyone::master.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('onlyone/master/' . $master->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/onlyone/master'))
                ->redirect();
        }

    }

    /**
     * Show master for editing.
     *
     * @param Request $request
     * @param Model   $master
     *
     * @return Response
     */
    public function edit(MasterRequest $request, Master $master)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('onlyone::master.name'))
            ->view('onlyone::master.edit', true)
            ->data(compact('master'))
            ->output();
    }

    /**
     * Update the master.
     *
     * @param Request $request
     * @param Model   $master
     *
     * @return Response
     */
    public function update(MasterRequest $request, Master $master)
    {
        try {
            $attributes = $request->all();

            $master->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('onlyone::master.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('onlyone/master/' . $master->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('onlyone/master/' . $master->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the master.
     *
     * @param Model   $master
     *
     * @return Response
     */
    public function destroy(MasterRequest $request, Master $master)
    {
        try {

            $master->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('onlyone::master.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('onlyone/master/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('onlyone/master/' . $master->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple master.
     *
     * @param Model   $master
     *
     * @return Response
     */
    public function delete(MasterRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('onlyone::master.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('onlyone/master'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/onlyone/master'))
                ->redirect();
        }

    }

    /**
     * Restore deleted masters.
     *
     * @param Model   $master
     *
     * @return Response
     */
    public function restore(MasterRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('onlyone::master.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/onlyone/master'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/onlyone/master/'))
                ->redirect();
        }

    }

}
